/*===========================================================================
 * Project         : hardware4linux
 * Module          : hwreport
 * File            : reportusb.c
 * Copyright       : (C) 2011 by Frederic Lepied
 * Author          : Frederic Lepied
 * Created On      : Tue May 17 22:41:30 2011
 * Purpose         : report usb ids
 *===========================================================================*/

#include <stdio.h>
#include <sys/types.h>

#include <libusb.h>

static void
print_devs(libusb_device **devs)
{
    libusb_device *dev;
    int i = 0;
    int loop, loop2;
    struct libusb_config_descriptor *config;
    
    while ((dev = devs[i++]) != NULL) {
	struct libusb_device_descriptor desc;
	int r = libusb_get_device_descriptor(dev, &desc);
	if (r < 0) {
	    fprintf(stderr, "failed to get device descriptor");
	    return;
	}

	r = libusb_get_active_config_descriptor(dev, &config);
	if (r != 0) {
	    fprintf(stderr, "failed to get active config descriptor");
	    return;
	}

	for(loop=0; loop<config[0].bNumInterfaces; loop++) {
	    for(loop2=0; loop2<config[0].interface[loop].num_altsetting; loop2++) {
		char embeded_manufacturer[256];
		char embeded_product[256];
		char *p;
		libusb_device_handle *handle;

		if (libusb_open(dev, &handle) == 0) {
		    if (libusb_get_string_descriptor_ascii(handle,
							   desc.iManufacturer,
							   embeded_manufacturer,
							   sizeof(embeded_manufacturer)) <= 0) {
			embeded_manufacturer[0] = '\0';
		    }
		    
		    if (libusb_get_string_descriptor_ascii(handle,
							   desc.iProduct,
							   embeded_product,
							   sizeof(embeded_product)) <= 0) {
			embeded_product[0] = '\0';
		    }
		    
		    libusb_close(handle);
		} else {
		    embeded_manufacturer[0] = '\0';
		    embeded_product[0] = '\0';		    
		}

		/* strip out any ':' to avoid problem in parsing on the server */
		p = embeded_manufacturer;
		while(*p) {
		    if (*p == ':') *p = '-';
		    p++;
		}
		p = embeded_product;
		while(*p) {
		    if (*p == ':') *p = '-';
		    p++;
		}
		
		printf("%04x:%04x:%02x:%02x:%02x:%s:%s\n",
		       desc.idVendor, desc.idProduct,
		       config[0].interface[loop].altsetting[loop2].bInterfaceClass,
		       config[0].interface[loop].altsetting[loop2].bInterfaceSubClass,
		       config[0].interface[loop].altsetting[loop2].bInterfaceProtocol,
		       embeded_manufacturer,
		       embeded_product);
	    }
	}
    }
}

int
main(void)
{
    libusb_device **devs;
    int r;
    ssize_t cnt;
    
    r = libusb_init(NULL);
    if (r < 0)
	return r;
    
    cnt = libusb_get_device_list(NULL, &devs);
    if (cnt < 0)
	return (int) cnt;
    
    print_devs(devs);
    libusb_free_device_list(devs, 1);
    
    libusb_exit(NULL);
    return 0;
}

/*
 * Local variables:
 * mode: c
 * End:
 *
 * reportusb.c ends here
 */
